@@ -1,16 +1,21 @@
 """
-Simulate ControlledDrivetrain.ssp and plot the results
+Example that demonstrates the simulation of SSPs (System Structure and Parameterization)
 """
 
 from fmpy.ssp.simulation import simulate_ssp
-from fmpy.util import plot_result, download_file
+from fmpy.util import download_file, plot_result
 
 
 def simulate_controlled_drivetrain(show_plot=True):
+    """ Download and simulate ControlledDrivetrain.ssp
+
+    Parameters:
+        show_plot     plot the results
+    """
 
     ssp_filename = r'ControlledDrivetrain.ssp'
 
-    download_file('https://github.com/CATIA-Systems/FMPy/releases/download/v0.0.9/' + ssp_filename, checksum='0af81ce9')
+    download_file('https://github.com/CATIA-Systems/FMPy/releases/download/v0.1.1/' + ssp_filename, checksum='45e667ed')
 
     print("Simulating %s..." % ssp_filename)
     result = simulate_ssp(ssp_filename, stop_time=4, step_size=1e-3)
